package org.diagbot.push.controller;

import org.diagbot.bigdata.work.AlgorithmCore;
import org.diagbot.common.work.FeatureRate;
import org.diagbot.common.work.ResponseData;
import org.diagbot.common.work.SearchData;
import org.diagbot.graphWeb.work.GraphCalculate;
import org.diagbot.pub.api.Response;
import org.diagbot.pub.utils.http.HttpApi;
import org.diagbot.pub.web.BaseController;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;

/**
 * @ClassName org.diagbot.push.controller.AlgorithmController
 * @Description TODO
 * @Author fyeman
 * @Date 2019/1/16/016 19:00
 * @Version 1.0
 **/
@Controller
@RequestMapping("/algorithm")
public class AlgorithmController extends BaseController {
    {
        listView = "/pages/algorithm/list.html";
    }

    @RequestMapping({"/index"})
    public String index() {
        return listView;
    }

    @RequestMapping(value = "/page_neural")
    @ResponseBody
    public Response<ResponseData> neuralData(HttpServletRequest request, SearchData searchData) throws Exception {
        return algorithm(request, searchData);
    }

    @RequestMapping(value = "/neural", method = RequestMethod.POST)
    @ResponseBody
    public Response<ResponseData> bayesPageData(HttpServletRequest request, @RequestBody SearchData searchData) throws Exception {
        return algorithm(request, searchData);
    }

    public Response<ResponseData> algorithm(HttpServletRequest request, SearchData searchData) throws Exception {
        Response<ResponseData> response = new Response();
        //大数据推送
        HttpApi<Response> api = new HttpApi<Response>();
        // 根据hospitalCode从his获取所有疾病库信息
        Response bigDataResponse = api.doPost("http://192.168.3.180:5001/bigdata-web/algorithm/neural", searchData,
                Response.class);
        LinkedHashMap bigDataLinkedHashMap = (LinkedHashMap) bigDataResponse.getData();
        //转FeatureRate对象
        ResponseData bigDataResponseData = map2ResponseData(bigDataLinkedHashMap);
        //知识图谱推送
        Response graphResponse = api.doPost("http://192.168.3.180:5003/graph-web/graph/push", searchData,
                Response.class);
        LinkedHashMap graphLinkedHashMap = (LinkedHashMap) graphResponse.getData();
        //转FeatureRate对象
        ResponseData graphResponseData = map2ResponseData(graphLinkedHashMap);
        if (graphResponseData.getDis().size() > 0) {
            List<FeatureRate> disFeatureRates = new ArrayList<>();
            boolean isFind = false;
            for (int i = 0; i < bigDataResponseData.getDis().size(); i++) {
                FeatureRate bigdata_fr = bigDataResponseData.getDis().get(i);
                isFind = false;
                for (FeatureRate graph_fr : graphResponseData.getDis()) {
                    if (bigdata_fr.getFeatureName().equals(graph_fr.getFeatureName())) {
                        isFind = true;
                    }
                }
                if (!isFind) {
                    disFeatureRates.add(bigdata_fr);
                }
            }
            List<FeatureRate> graphFeatureRates = graphResponseData.getDis();
            graphFeatureRates.addAll(disFeatureRates);
            //大数据推送疾病数据用知识图谱替换
            bigDataResponseData.setDis(graphFeatureRates);
        }
        response.setData(bigDataResponseData);
        return response;
    }

    private ResponseData map2ResponseData(LinkedHashMap bigDataLinkedHashMap) {
        ResponseData responseData = new ResponseData();
        if (bigDataLinkedHashMap.get("dis") != null) {
            responseData.setDis(map2FeatureRates((List<LinkedHashMap<String, String>>)bigDataLinkedHashMap.get("dis")));
        }
        if (bigDataLinkedHashMap.get("symptom") != null) {
            responseData.setSymptom(map2FeatureRates((List<LinkedHashMap<String, String>>)bigDataLinkedHashMap.get("symptom")));
        }
        if (bigDataLinkedHashMap.get("vitals") != null) {
            responseData.setVitals(map2FeatureRates((List<LinkedHashMap<String, String>>)bigDataLinkedHashMap.get("vitals")));
        }
        if (bigDataLinkedHashMap.get("labs") != null) {
            responseData.setLabs(map2FeatureRates((List<LinkedHashMap<String, String>>)bigDataLinkedHashMap.get("labs")));
        }
        if (bigDataLinkedHashMap.get("pacs") != null) {
            responseData.setPacs(map2FeatureRates((List<LinkedHashMap<String, String>>)bigDataLinkedHashMap.get("pacs")));
        }
        return responseData;
    }

    private List<FeatureRate> map2FeatureRates(List<LinkedHashMap<String, String>> data) {
        List<FeatureRate> featureRates = new ArrayList<>();
        for (LinkedHashMap<String, String> map : data) {
            FeatureRate fr = new FeatureRate();
            fr.setFeatureName(map.get("featureName"));
            fr.setRate(map.get("rate"));
            fr.setExtraProperty(map.get("extraProperty"));
            featureRates.add(fr);
        }
        return featureRates;
    }
}
